<?php
/**
 * @package   admintoolswp
 * @copyright Copyright (c)2017-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL version 3 or later
 */

namespace Akeeba\AdminTools\Library\Geoip;

use Akeeba\AdminTools\Admin\Helper\Language;
use Akeeba\AdminTools\Library\Transfer\Transfer;
use GeoIp2\Database\Reader;
use GeoIp2\Exception\AddressNotFoundException;
use MaxMind\Db\Reader\InvalidDatabaseException;

defined('ADMINTOOLSINC') or die;

class Akgeoip
{
	/** @var	\GeoIp2\Database\Reader	The MaxMind GeoLite database reader */
	private $reader = null;

	/** @var	array	Records for IP addresses already looked up */
	private $lookups = array();

	/**
	 * Public constructor. Loads up the GeoLite2 database.
	 */
	public function __construct()
	{
		if (!function_exists('bcadd') || !function_exists('bcmul') || !function_exists('bcpow'))
		{
			require_once __DIR__ . '/fakebcmath.php';
		}

		require_once __DIR__.'/vendor/autoload.php';

		$filePath = __DIR__ . '/../../assets/akgeoip/db/GeoLite2-Country.mmdb';

		try
		{
			$this->reader = new Reader($filePath);
		}
		// If anything goes wrong, MaxMind will raise an exception, resulting in a WSOD. Let's be sure to catch everything
		catch(\Exception $e)
		{
			$this->reader = null;
		}
	}

	/**
	 * Gets a raw country record from an IP address
	 *
	 * @param   string  $ip  The IP address to look up
	 *
	 * @return  mixed  A \GeoIp2\Model\Country record if found, false if the IP address is not found, null if the db can't be loaded
	 */
	public function getCountryRecord($ip)
	{
		if (!array_key_exists($ip, $this->lookups))
		{
			try
			{
				if(!is_null($this->reader))
				{
					$this->lookups[$ip] = $this->reader->country($ip);
				}
				else
				{
					$this->lookups[$ip] = null;
				}
			}
			catch (AddressNotFoundException $e)
			{
				$this->lookups[$ip] = false;
			}
			catch (InvalidDatabaseException $e)
			{
				$this->lookups[$ip] = null;
			}
            // GeoIp2 could throw several different types of exceptions. Let's be sure that we're going to catch them all
            catch (\Exception $e)
            {
                $this->lookups[$ip] = null;
            }

		}

		return $this->lookups[$ip];
	}

	/**
	 * Gets the ISO country code from an IP address
	 *
	 * @param   string  $ip  The IP address to look up
	 *
	 * @return  mixed  A string with the country ISO code if found, false if the IP address is not found, null if the db can't be loaded
	 */
	public function getCountryCode($ip)
	{
		$record = $this->getCountryRecord($ip);

		if ($record === false)
		{
			return false;
		}
		elseif (is_null($record))
		{
			return false;
		}
		else
		{
			return $record->country->isoCode;
		}
	}

	/**
	 * Gets the country name from an IP address
	 *
	 * @param   string  $ip      The IP address to look up
	 * @param   string  $locale  The locale of the country name, e.g 'de' to return the country names in German. If not specified the English (US) names are returned.
	 *
	 * @return  mixed  A string with the country name if found, false if the IP address is not found, null if the db can't be loaded
	 */
	public function getCountryName($ip, $locale = null)
	{
		$record = $this->getCountryRecord($ip);

		if ($record === false)
		{
			return false;
		}
		elseif (is_null($record))
		{
			return false;
		}
		else
		{
			if (empty($locale))
			{
				return $record->country->name;
			}
			else
			{
				return $record->country->names[$locale];
			}
		}
	}

	/**
	 * Gets the continent ISO code from an IP address
	 *
	 * @param   string  $ip      The IP address to look up
	 *
	 * @return  mixed  A string with the country name if found, false if the IP address is not found, null if the db can't be loaded
	 */
	public function getContinent($ip)
	{
		$record = $this->getCountryRecord($ip);

		if ($record === false)
		{
			return false;
		}
		elseif (is_null($record))
		{
			return false;
		}
		else
		{
			return $record->continent->code;
		}
	}

	/**
	 * Gets the continent name from an IP address
	 *
	 * @param   string  $ip      The IP address to look up
	 * @param   string  $locale  The locale of the continent name, e.g 'de' to return the country names in German. If not specified the English (US) names are returned.
	 *
	 * @return  mixed  A string with the country name if found, false if the IP address is not found, null if the db can't be loaded
	 */
	public function getContinentName($ip, $locale = null)
	{
		$record = $this->getCountryRecord($ip);

		if ($record === false)
		{
			return false;
		}
		elseif (is_null($record))
		{
			return false;
		}
		else
		{
			if (empty($locale))
			{
				return $record->continent;
			}
			else
			{
				return $record->continent->names[$locale];
			}
		}
	}

	/**
	 * Downloads and installs a fresh copy of the GeoLite2 Country database
	 *
	 * @return  mixed  True on success, error string on failure
	 */
	public function updateDatabase()
	{
		$datFile = ADMINTOOLSWP_PATH.'/app/assets/akgeoip/db/GeoLite2-Country.mmdb';

		// Sanity check
		if(!function_exists('gzinflate'))
		{
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_NOGZSUPPORT');
		}

		// Try to download the package, if I get any exception I'll simply stop here and display the error
		try
		{
			$compressed = $this->downloadDatabase();
		}
		catch(\Exception $e)
		{
			return $e->getMessage();
		}

		// Write the downloaded file to a temporary location
		$tmpdir = ADMINTOOLSWP_TMP;
		$target = $tmpdir.'/GeoLite2-Country.mmdb.gz';

		$ret = file_put_contents($target, $compressed);

		if ($ret === false)
		{
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_WRITEFAILED');
		}

		unset($compressed);

		// Decompress the file
		$uncompressed = '';

		$zp = @gzopen($target, 'rb');

		if($zp !== false)
		{
			while(!gzeof($zp))
			{
				$uncompressed .= @gzread($zp, 102400);
			}

			@gzclose($zp);
			@unlink($target);
		}
		else
		{
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_CANTUNCOMPRESS');
		}

		// Double check if MaxMind can actually read and validate the downloaded database
		try
		{
			// The Reader want a file, so let me write again the file in the temp directory
			file_put_contents($target, $uncompressed);
			new Reader($target);
		}
		catch(\Exception $e)
		{
			@unlink($target);
			// MaxMind could not validate the database, let's inform the user
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_INVALIDDB');
		}

		unlink($target);


		// Check the size of the uncompressed data. When MaxMind goes into overload, we get crap data in return.
		if (strlen($uncompressed) < 1048576)
		{
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_MAXMINDRATELIMIT');
		}

		// Check the contents of the uncompressed data. When MaxMind goes into overload, we get crap data in return.
		if (stristr($uncompressed, 'Rate limited exceeded') !== false)
		{
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_MAXMINDRATELIMIT');
		}

		// Remove old file

		if (file_exists($datFile))
		{
			if(!unlink($datFile))
			{
				return Language::_('PLG_SYSTEM_AKGEOIP_ERR_CANTDELETEOLD');
			}
		}

		// Write the update file
		if (!file_put_contents($datFile, $uncompressed))
		{
			return Language::_('PLG_SYSTEM_AKGEOIP_ERR_CANTWRITE');
		}

		return true;
	}

	private function downloadDatabase()
	{
		// Download the latest MaxMind GeoCountry Lite database
		$url = 'http://geolite.maxmind.com/download/geoip/database/GeoLite2-Country.mmdb.gz';

		$http = new Transfer();

		// If I am using curl, let's store and send cookies, it should be more fail-proof against CloudFlare security
		if($http->getAdapterName() == 'curl')
		{
			$cookiefile = tempnam(ADMINTOOLSWP_TMP, 'geoip');
			$cookiejar  = tempnam(ADMINTOOLSWP_TMP, 'geoip');

			$http->setAdapterOptions(array(
				CURLOPT_COOKIEFILE => $cookiefile,
				CURLOPT_COOKIEJAR  => $cookiejar
			));
		}

		$compressed = $http->getFromURL($url);

		// Remove cookie files, we don't need them
		if($http->getAdapterName() == 'curl')
		{
			unlink($cookiefile);
			unlink($cookiejar);
		}

		// An empty file indicates a problem with MaxMind's servers
		if (empty($compressed))
		{
			throw new \Exception(Language::_('PLG_SYSTEM_AKGEOIP_ERR_EMPTYDOWNLOAD'));
		}

		// Sometimes you get a rate limit exceeded
		if (stristr($compressed, 'Rate limited exceeded') !== false)
		{
			throw new \Exception(Language::_('PLG_SYSTEM_AKGEOIP_ERR_MAXMINDRATELIMIT'));
		}

		return $compressed;
	}
}
